require 'spec_helper'

describe 'User - Group Relationship' do
  let(:user) { User.create(name: 'Marius') }
  let(:group) { Group.create(name: 'Calculus') }

  it 'group is subscribable' do
    expect( Group.subscribable? ).to be_true
  end

  it 'user is subscribable' do
    expect( User.subscribable? ).to be_true
  end

  it 'user can subscribe to another user' do
    user.subscribe_to!(user)
    expect(user.subscribers.size).to eq(1)
  end

  it 'group can subscribe to another group' do
    group.subscribe_to!(group)
    expect(group.subscribers.size).to eq(1)
  end

  context 'uniqueness' do
    it 'allows saving subscribable' do
      sub = user.subscriptions.new
      sub.subscriber = group
      expect{ sub.save}.to_not raise_error
    end

    it 'does not allow to save duplicate subscribable' do
      sub = user.subscriptions.new
      sub.subscription = group
      sub.save

      sub2 = user.subscriptions.new
      sub2.subscription = group
      expect{ sub2.save }.to raise_error(ActiveRecord::RecordNotUnique)
    end
  end

  context 'subscribed_to?' do
    it 'returns true if user is subscribed to group' do
      sub = user.subscriptions.new
      sub.subscription = group
      sub.save
      expect(user.subscribed_to?(group)).to be_true
    end

    it 'returns false if user is not subscribed to group' do
      expect(user.subscribed_to?(group)).to be_false
    end
  end

  it 'can delete a subscription to a group' do
    user.subscribe_to!(group)
    user.delete_subscription_to!(group)
    expect(user.subscribed_to?(group)).to be_false
  end

  context 'subscribers' do
    it 'can search by type' do
      group.subscribe_to!(user)
      user.subscribe_to!(user)
      expect(user.subscribers_from('Group').size).to eq(1)
    end

    it 'can search by many types' do
      group.subscribe_to!(user)
      user.subscribe_to!(user)
      expect(user.subscribers_from('User', 'Group').size).to eq(2)
    end
  end

  context 'subscriptions' do
    it 'can search by type' do
      user.subscribe_to!(group)
      user.subscribe_to!(user)
      expect(user.subscriptions_to('Group').size).to eq(1)
    end
    it 'can search by many type' do
      user.subscribe_to!(group)
      user.subscribe_to!(user)
      expect(user.subscriptions_to('Group', 'User').size).to eq(2)
    end
  end
end
